// Header File
// Created 09/06/2006; 20:27:57
#ifndef CRI_3D_H
#define CRI_3D_H

#define MAX_POINTS 64
#define MAX_CONNECTIONS 32
#define MAX_FACES 32

typedef struct {
	short x, y ;
} Coordinate ;

typedef Coordinate Coordinates [] ;

typedef struct {
	short width, height ;
	char * data ;
} Texture ;

typedef struct {
	void  * address ;
	short width, height ;				/* Unused fields so far	*/
} Screen ;

Screen LCDScreen ;

typedef struct {
	long  rx, ry, rz ;
	short tx, ty, tz ;
	short zoomx, zoomy, focal ; // This is floating number (the upper 8 bits represent the integer value, whereas the lower 8 bits represent the decimal part)
} Camera ;

typedef struct {
	long  npoints ;
	long  points [MAX_POINTS*3] ;
	long  nconnections ;
	short connections [MAX_CONNECTIONS*2] ;
	long  nnfaces ;			/* Number of number decorated faces */
	short nfaces [MAX_FACES] ;	/* Array of number of decorated faces */
	short faces [MAX_FACES*3] ; /* Average of faces: 4, plus 4 mapping coordinate */
	Coordinate mappingcoordinates [MAX_FACES*3] ; /* Average of faces: 4, plus 4 mapping coordinate */
	Texture * textures [MAX_FACES] ;
} Scene ;

Scene * CreateScene () ;

void InitCri3D () ;

void ReleaseCri3D () ;

void ClearScene (const Scene * scene) ;

void AddPoint (const Scene * scene, long x, long y, long z) ;

void ClearPoints (const Scene * scene) ;

void AddConnection (const Scene * scene, short point1, short point2) ;

void ClearConnections (const Scene * scene) ;

/**
 *  Add a face with no texture
 */
void AddFace (const Scene * scene, short npoints, short points []) ;

/**
 * Add a face decorated with a texture
 * @param npoints Number of points concerned by the face
 * @param points  List of points describing the face (they must be described in a circular
 *        geometric order to be displayed). The array size must match [npoints].
 * @param mcoords mappings coordinates. The array dimension must match [npoints].
 */
void AddDecoredFace (const Scene * scene, const Texture * t, short npoints, short points [], Coordinate mcoords []) ;

/**
 * Calculates the transformation
 */
void Transform (const Camera * camera, const Scene * scene) ;

/**
 * Render the scene on the screen. Transform operation must be previously called
 */
void Render (const Screen * screen, const Camera * camera, const Scene * scene) ;

void DrawPolygon (const Screen * screen, short nb, Coordinate coords []) ;

void FillPolygon (const Screen * screen, short nb, Coordinate coords []) ;

/**
 * Returns true if the coordinates appears in the opposite order of the clockwise one.
 * This operation is used to know which side of the polygon is drawn (and thus if it
 * has to be rendered or not).
 */
char IsAntiClockwise (const Scene * scene, short face) ;

/**
 * Draw a texture inside the triangle defined by the three coordinates.
 * Texture width must be < 128.
 */
void FillPolygonWithTexture (const Screen * screen, const Texture * t,
															short nb,
															const Coordinate coords [],
															const Coordinate mappings []) ;
/**
 * Draw a texture inside the triangle defined by the three coordinates.
 * Texture width must be < 128.
 */
void FillTriangleWithTexture (const Screen * screen, const Texture * t,
															const Coordinate * c0, const Coordinate * map0,
															const Coordinate * c1, const Coordinate * map1,
															const Coordinate * c2, const Coordinate * map2) ;

/**
 * Convert a screen into a texture.
 * A Screen use one bit per pixel, whereas the texture uses one byte per pixel, for
 * optimisations purposes.
 * The copy size ONLY depends on texture width and height. Ensure though that teh screen
 * width and height are not zero.
 * Warning: The texture data must be allocated before calling this function.
 */
void Screen2Texture (const Screen * screen, const Texture * texture) ;

/**
 * Ultra fast calculation of the sinus, using precalculed table, and using integer instead of float.
 * Warning: The measure is not degrees, use the table bellow for correspondance between degrees, and
 *          measure given in parameter
 *          360 <=> 128
 *          180 <=> 64
 *          90  <=> 32
 */
short Sinus (short angle) ;

/**
 * Ultra fast calculation of the cosinus, using precalculed table, and using integer instead of float.
 * Warning: The measure is not degrees, use the table bellow for correspondance between degrees, and
 *          measure given in parameter
 *          360 <=> 128
 *          180 <=> 64
 *          90  <=> 32
 */
short Cosinus (short angle) ;

/**
 * Change mapping coordinates for the given scene
 */
void SetMappingCoordinate (Scene * scene, short face, Coordinate mcoords []) ;

/**
 * Use RLE encoding to decode a 1 bit per pixel picture. The first value indicates the amount of
 * NULL/0 values.
 * The returned value is an allocated array of bytes.
 * Ensures to de-allocate it when exiting.
 */
char * Decode (const short width, const short height, const short picture []) ;

#endif